
/* Chrysalide - Outil d'analyse de fichiers binaires
 * manager.h - prototypes pour la collecte et la gestion des instances partagées
 *
 * Copyright (C) 2016-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ARCH_SHARING_MANAGER_H
#define _ARCH_SHARING_MANAGER_H


#include <glib-object.h>
#include <stdbool.h>


#include "container.h"
#include "instance.h"



/* Compare une instance avec des informations similaires. */
typedef gboolean (* compare_with_info_fc) (const GSharedInstance *, const void *);


#define G_TYPE_SHARE_MANAGER            (g_share_manager_get_type())
#define G_SHARE_MANAGER(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), G_TYPE_SHARE_MANAGER, GShareManager))
#define G_SHARE_MANAGER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_SHARE_MANAGER, GShareManagerClass))
#define G_IS_SHARE_MANAGER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), G_TYPE_SHARE_MANAGER))
#define G_IS_SHARE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_SHARE_MANAGER))
#define G_SHARE_MANAGER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_SHARE_MANAGER, GShareManagerClass))


/* Gestionnaire d'instances de type identique partagées (instance) */
typedef struct _GShareManager GShareManager;

/* Gestionnaire d'instances de type identique partagées (classe) */
typedef struct _GShareManagerClass GShareManagerClass;


/* Détermine le type du gestionnaire d'instances partagées. */
GType g_share_manager_get_type(void);

/* Crée un nouveau gestionnaire d'instances partagées. */
GShareManager *g_share_manager_new(GType);

/* Retrouve ou crée une instance partagée. */
GSharedInstance *g_share_manager_get(GShareManager *, GSharedInstance *);

/* Met à jour une instance partagée. */
GSharedInstance *g_share_manager_update(GShareManager *, GSharedInstance *, GSharedInstance *, GShareContainer *);

/* Abandonne un usage d'une instance partagée. */
void g_share_manager_put(GShareManager *, GSharedInstance *);

/* Imprime des statistiques d'utilisation du gestionnaire. */
#ifdef DEBUG_DUMP_STATS
void g_share_manager_dump_stats(GShareManager *);
#endif



#endif  /* _ARCH_SHARING_MANAGER_H */
